@@ -63,10 +63,12 @@ class Runner:
         if exclude_paths is None:
             exclude_paths = []
 
-        # Assure consistent type
+        # Assure consistent type and configure given lintables as explicit (so
+        # excludes paths would not apply on them).
         for item in lintables:
             if not isinstance(item, Lintable):
                 item = Lintable(item)
+            item.explicit = True
             self.lintables.add(item)
 
         # Expand folders (roles) to their components
@@ -99,6 +101,11 @@ class Runner:
 
         # Exclusions should be evaluated only using absolute paths in order
         # to work correctly.
+
+        # Explicit lintables are never excluded
+        if lintable.explicit:
+            return False
+
         abs_path = str(lintable.abspath)
         if self.project_dir and not abs_path.startswith(self.project_dir):
             _logger.debug(
@@ -147,7 +154,6 @@ class Runner:
                         )
                     else:
                         filename = warn.source
-                        # breakpoint()
                         match = MatchError(
                             message=warn.message
                             if isinstance(warn.message, str)
@@ -166,7 +172,7 @@ class Runner:
                 )
         return matches
 
-    def _run(self) -> list[MatchError]:  # noqa: C901
+    def _run(self) -> list[MatchError]:
         """Run the linting (inner loop)."""
         files: list[Lintable] = []
         matches: list[MatchError] = []
@@ -189,6 +195,16 @@ class Runner:
                     ),
                 )
                 lintable.stop_processing = True
+            # identify missing files/folders
+            if not lintable.path.exists():
+                matches.append(
+                    MatchError(
+                        lintable=lintable,
+                        message="File or directory not found.",
+                        rule=LoadingFailureRule(),
+                        tag="load-failure[not-found]",
+                    ),
+                )
 
         # -- phase 1 : syntax check in parallel --
         def worker(lintable: Lintable) -> list[MatchError]:
@@ -242,7 +258,7 @@ class Runner:
         # remove any matches made inside excluded files
         matches = list(
             filter(
-                lambda match: not self.is_excluded(Lintable(match.filename))
+                lambda match: not self.is_excluded(match.lintable)
                 and hasattr(match, "lintable")
                 and match.tag not in match.lintable.line_skips[match.lineno],
                 matches,
